"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LlamaContext = void 0;
Object.defineProperty(exports, "SchemaGrammarConverter", {
  enumerable: true,
  get: function () {
    return _grammar.SchemaGrammarConverter;
  }
});
Object.defineProperty(exports, "convertJsonSchemaToGrammar", {
  enumerable: true,
  get: function () {
    return _grammar.convertJsonSchemaToGrammar;
  }
});
exports.initLlama = initLlama;
exports.releaseAllLlama = releaseAllLlama;
exports.setContextLimit = setContextLimit;
var _reactNative = require("react-native");
var _NativeRNLlama = _interopRequireDefault(require("./NativeRNLlama"));
var _grammar = require("./grammar");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const EVENT_ON_TOKEN = '@RNLlama_onToken';
const EventEmitter =
// @ts-ignore
new _reactNative.NativeEventEmitter(_NativeRNLlama.default);
class LlamaContext {
  gpu = false;
  reasonNoGPU = '';
  constructor(_ref) {
    let {
      contextId,
      gpu,
      reasonNoGPU
    } = _ref;
    this.id = contextId;
    this.gpu = gpu;
    this.reasonNoGPU = reasonNoGPU;
  }
  async completion(params, callback) {
    let tokenListener = EventEmitter.addListener(EVENT_ON_TOKEN, evt => {
      const {
        contextId,
        tokenResult
      } = evt;
      if (contextId !== this.id) return;
      callback(tokenResult);
    });
    const promise = _NativeRNLlama.default.completion(this.id, params);
    return promise.then(completionResult => {
      tokenListener.remove();
      tokenListener = null;
      return completionResult;
    }).catch(err => {
      tokenListener.remove();
      tokenListener = null;
      throw err;
    });
  }
  stopCompletion() {
    return _NativeRNLlama.default.stopCompletion(this.id);
  }
  tokenize(text) {
    return _NativeRNLlama.default.tokenize(this.id, text);
  }
  detokenize(tokens) {
    return _NativeRNLlama.default.detokenize(this.id, tokens);
  }
  embedding(text) {
    return _NativeRNLlama.default.embedding(this.id, text);
  }
  async release() {
    return _NativeRNLlama.default.releaseContext(this.id);
  }
}
exports.LlamaContext = LlamaContext;
async function setContextLimit(limit) {
  return _NativeRNLlama.default.setContextLimit(limit);
}
async function initLlama(_ref2) {
  let {
    model,
    is_model_asset: isModelAsset,
    ...rest
  } = _ref2;
  let path = model;
  if (path.startsWith('file://')) path = path.slice(7);
  const {
    contextId,
    gpu,
    reasonNoGPU
  } = await _NativeRNLlama.default.initContext({
    model: path,
    is_model_asset: !!isModelAsset,
    ...rest
  });
  return new LlamaContext({
    contextId,
    gpu,
    reasonNoGPU
  });
}
async function releaseAllLlama() {
  return _NativeRNLlama.default.releaseAllContexts();
}
//# sourceMappingURL=index.js.map