import { NativeEventEmitter } from 'react-native';
import RNLlama from './NativeRNLlama';
import { SchemaGrammarConverter, convertJsonSchemaToGrammar } from './grammar';
export { SchemaGrammarConverter, convertJsonSchemaToGrammar };
const EVENT_ON_TOKEN = '@RNLlama_onToken';
const EventEmitter =
// @ts-ignore
new NativeEventEmitter(RNLlama);
export class LlamaContext {
  gpu = false;
  reasonNoGPU = '';
  constructor(_ref) {
    let {
      contextId,
      gpu,
      reasonNoGPU
    } = _ref;
    this.id = contextId;
    this.gpu = gpu;
    this.reasonNoGPU = reasonNoGPU;
  }
  async completion(params, callback) {
    let tokenListener = EventEmitter.addListener(EVENT_ON_TOKEN, evt => {
      const {
        contextId,
        tokenResult
      } = evt;
      if (contextId !== this.id) return;
      callback(tokenResult);
    });
    const promise = RNLlama.completion(this.id, params);
    return promise.then(completionResult => {
      tokenListener.remove();
      tokenListener = null;
      return completionResult;
    }).catch(err => {
      tokenListener.remove();
      tokenListener = null;
      throw err;
    });
  }
  stopCompletion() {
    return RNLlama.stopCompletion(this.id);
  }
  tokenize(text) {
    return RNLlama.tokenize(this.id, text);
  }
  detokenize(tokens) {
    return RNLlama.detokenize(this.id, tokens);
  }
  embedding(text) {
    return RNLlama.embedding(this.id, text);
  }
  async release() {
    return RNLlama.releaseContext(this.id);
  }
}
export async function setContextLimit(limit) {
  return RNLlama.setContextLimit(limit);
}
export async function initLlama(_ref2) {
  let {
    model,
    is_model_asset: isModelAsset,
    ...rest
  } = _ref2;
  let path = model;
  if (path.startsWith('file://')) path = path.slice(7);
  const {
    contextId,
    gpu,
    reasonNoGPU
  } = await RNLlama.initContext({
    model: path,
    is_model_asset: !!isModelAsset,
    ...rest
  });
  return new LlamaContext({
    contextId,
    gpu,
    reasonNoGPU
  });
}
export async function releaseAllLlama() {
  return RNLlama.releaseAllContexts();
}
//# sourceMappingURL=index.js.map